@@ -2,7 +2,8 @@
 #
 # Packager - A class for writing the Excel XLSX Worksheet file.
 #
-# Copyright 2013-2021, John McNamara, jmcnamara@cpan.org
+# SPDX-License-Identifier: BSD-2-Clause
+# Copyright 2013-2022, John McNamara, jmcnamara@cpan.org
 #
 
 # Standard packages.
@@ -11,7 +12,7 @@ import stat
 import tempfile
 from shutil import copy
 
-from .compatibility import StringIO
+from io import StringIO
 from io import BytesIO
 
 # Package imports.
