@@ -2,21 +2,23 @@
 #
 # Workbook - A class for writing the Excel XLSX Workbook file.
 #
-# Copyright 2013-2021, John McNamara, jmcnamara@cpan.org
+# SPDX-License-Identifier: BSD-2-Clause
+# Copyright 2013-2022, John McNamara, jmcnamara@cpan.org
 #
 
 # Standard packages.
 import hashlib
-import re
-import os
 import operator
+import os
+import re
 import time
-from warnings import warn
 from datetime import datetime
-from zipfile import ZipFile, ZipInfo, ZIP_DEFLATED, LargeZipFile
+from decimal import Decimal
+from fractions import Fraction
 from struct import unpack
+from warnings import warn
+from zipfile import ZipFile, ZipInfo, ZIP_DEFLATED, LargeZipFile
 
-from .compatibility import int_types, num_types, str_types, force_unicode
 
 # Package imports.
 from . import xmlwriter
@@ -296,8 +298,7 @@ class Workbook(xmlwriter.XMLwriter):
 
         """
         if not is_stream and not os.path.exists(vba_project):
-            warn("VBA project binary file '%s' not found."
-                 % force_unicode(vba_project))
+            warn("VBA project binary file '%s' not found." % vba_project)
             return -1
 
         if self.vba_codename is None:
@@ -323,7 +324,7 @@ class Workbook(xmlwriter.XMLwriter):
                 self._store_workbook()
             except IOError as e:
                 raise FileCreateError(e)
-            except LargeZipFile as e:
+            except LargeZipFile:
                 raise FileSizeError("Filesize would require ZIP64 extensions. "
                                     "Use workbook.use_zip64().")
 
@@ -416,9 +417,9 @@ class Workbook(xmlwriter.XMLwriter):
                 property_type = 'bool'
             elif isinstance(value, datetime):
                 property_type = 'date'
-            elif isinstance(value, int_types):
+            elif isinstance(value, int):
                 property_type = 'number_int'
-            elif isinstance(value, num_types):
+            elif isinstance(value, (float, int, Decimal, Fraction)):
                 property_type = 'number'
             else:
                 property_type = 'text'
@@ -428,13 +429,11 @@ class Workbook(xmlwriter.XMLwriter):
 
         if property_type == 'text' and len(value) > 255:
             warn("Length of 'value' parameter exceeds Excel's limit of 255 "
-                 "characters in set_custom_property(): '%s'" %
-                 force_unicode(value))
+                 "characters in set_custom_property(): '%s'" % value)
 
         if len(name) > 255:
             warn("Length of 'name' parameter exceeds Excel's limit of 255 "
-                 "characters in set_custom_property(): '%s'" %
-                 force_unicode(name))
+                 "characters in set_custom_property(): '%s'" % name)
 
         self.custom_properties.append((name, value, property_type))
 
@@ -495,8 +494,7 @@ class Workbook(xmlwriter.XMLwriter):
 
             # Warn if the sheet index wasn't found.
             if sheet_index is None:
-                warn("Unknown sheet name '%s' in defined_name()"
-                     % force_unicode(sheetname))
+                warn("Unknown sheet name '%s' in defined_name()" % sheetname)
                 return -1
         else:
             # Use -1 to indicate global names.
@@ -505,21 +503,19 @@ class Workbook(xmlwriter.XMLwriter):
         # Warn if the defined name contains invalid chars as defined by Excel.
         if (not re.match(r'^[\w\\][\w\\.]*$', name, re.UNICODE)
                 or re.match(r'^\d', name)):
-            warn("Invalid Excel characters in defined_name(): '%s'"
-                 % force_unicode(name))
+            warn("Invalid Excel characters in defined_name(): '%s'" % name)
             return -1
 
         # Warn if the defined name looks like a cell name.
         if re.match(r'^[a-zA-Z][a-zA-Z]?[a-dA-D]?[0-9]+$', name):
-            warn("Name looks like a cell name in defined_name(): '%s'"
-                 % force_unicode(name))
+            warn("Name looks like a cell name in defined_name(): '%s'" % name)
             return -1
 
         # Warn if the name looks like a R1C1 cell reference.
         if (re.match(r'^[rcRC]$', name)
                 or re.match(r'^[rcRC]\d+[rcRC]\d+$', name)):
             warn("Invalid name '%s' like a RC cell ref in defined_name()"
-                 % force_unicode(name))
+                 % name)
             return -1
 
         self.defined_names.append([name, sheet_index, formula, False])
@@ -937,7 +933,7 @@ class Workbook(xmlwriter.XMLwriter):
             num_format = xf_format.num_format
 
             # Check if num_format is an index to a built-in number format.
-            if not isinstance(num_format, str_types):
+            if not isinstance(num_format, str):
                 num_format = int(num_format)
 
                 # Number format '0' is indexed as 1 in Excel.
@@ -1353,25 +1349,20 @@ class Workbook(xmlwriter.XMLwriter):
         x_dpi = 96
         y_dpi = 96
 
-        # Look for numbers rather than strings for Python 2.6/3 compatibility.
-        marker_ihdr = 0x49484452  # IHDR
-        marker_phys = 0x70485973  # pHYs
-        marker_iend = 0x49454E44  # IEND
-
         # Search through the image data to read the height and width in the
         # IHDR element. Also read the DPI in the pHYs element.
         while not end_marker and offset < data_length:
 
             length = unpack('>I', data[offset + 0:offset + 4])[0]
-            marker = unpack('>I', data[offset + 4:offset + 8])[0]
+            marker = unpack('4s', data[offset + 4:offset + 8])[0]
 
             # Read the image dimensions.
-            if marker == marker_ihdr:
+            if marker == b'IHDR':
                 width = unpack('>I', data[offset + 8:offset + 12])[0]
                 height = unpack('>I', data[offset + 12:offset + 16])[0]
 
             # Read the image DPI.
-            if marker == marker_phys:
+            if marker == b'pHYs':
                 x_density = unpack('>I', data[offset + 8:offset + 12])[0]
                 y_density = unpack('>I', data[offset + 12:offset + 16])[0]
                 units = unpack('b', data[offset + 16:offset + 17])[0]
@@ -1380,7 +1371,7 @@ class Workbook(xmlwriter.XMLwriter):
                     x_dpi = x_density * 0.0254
                     y_dpi = y_density * 0.0254
 
-            if marker == marker_iend:
+            if marker == b'IEND':
                 end_marker = True
                 continue
 
@@ -1666,8 +1657,7 @@ class Workbook(xmlwriter.XMLwriter):
                 # in a chart series formula.
                 if sheetname not in worksheets:
                     warn("Unknown worksheet reference '%s' in range "
-                         "'%s' passed to add_series()"
-                         % (force_unicode(sheetname), force_unicode(c_range)))
+                         "'%s' passed to add_series()" % (sheetname, c_range))
                     chart.formula_data[r_id] = []
                     seen_ranges[c_range] = []
                     continue
